@@ -87,6 +87,9 @@ impl Chunk {
 pub(crate) struct Loader {
     cache: kismet_cache::Cache,
     remote_sources: Vec<Bucket>,
+    // These chunks will be returned without involving any cache nor
+    // `remote_sources`.
+    known_chunks: HashMap<Fingerprint, Arc<Chunk>>,
 }
 
 lazy_static::lazy_static! {
@@ -125,6 +128,28 @@ lazy_static::lazy_static! {
     };
 }
 
+/// Returns the fingerprint for the zero-filled chunk.  We have
+/// special knowledge of this chunk and know its contents without
+/// fetching it from remote storage.
+#[inline]
+pub(crate) fn zero_fingerprint() -> Fingerprint {
+    ZERO_FILLED_CHUNK.0
+}
+
+/// Returns whether `fp` is "known" to the chunk loader: we don't need
+/// to hit remote storage to figure out what data they refer to.
+/// Changing the set of well-known fingerprints may affect
+/// compatibility.
+#[inline]
+pub(crate) fn is_well_known_fingerprint(fp: Fingerprint) -> bool {
+    fp == zero_fingerprint()
+}
+
+/// Returns a vector of all well-known chunks.
+fn well_known_chunks() -> Vec<Arc<Chunk>> {
+    vec![ZERO_FILLED_CHUNK.1.clone()]
+}
+
 impl Chunk {
     fn new(fprint: Fingerprint, payload: memmap2::Mmap) -> Result<Chunk> {
         // The contents of a content-addressed chunk must have the same
@@ -148,6 +173,11 @@ impl Chunk {
         Ok(Chunk { fprint, payload })
     }
 
+    /// Returns a cached chunk for `fprint`, if we have one.
+    pub(crate) fn arc_from_cache(fprint: Fingerprint) -> Option<Arc<Chunk>> {
+        fetch_from_cache(fprint)
+    }
+
     /// Returns a chunk (potentially cached) from `payload`.
     pub(crate) fn arc_from_bytes(payload: &[u8]) -> Arc<Chunk> {
         let fprint = fingerprint_file_chunk(payload);
@@ -303,9 +333,21 @@ impl Loader {
         Ok(Loader {
             cache: cache_builder.build(),
             remote_sources,
+            known_chunks: well_known_chunks()
+                .into_iter()
+                .map(|chunk| (chunk.fprint(), chunk))
+                .collect(),
         })
     }
 
+    /// Registers a new chunk as "known" to the `Loader`, unless there
+    /// already was a known chunk with that fingerprint.
+    ///
+    /// Known chunks are returned directly and bypass any I/O.
+    pub(crate) fn adjoin_known_chunk(&mut self, chunk: Arc<Chunk>) {
+        self.known_chunks.entry(chunk.fprint()).or_insert(chunk);
+    }
+
     /// Attempts to fetch the chunk for each fingerprint in `fprints`.
     ///
     /// Return `Err` if any fetch fails.  Even on success, the return
@@ -346,8 +388,8 @@ impl Loader {
     pub(crate) fn fetch_chunk(&self, fprint: Fingerprint) -> Result<Option<Arc<Chunk>>> {
         use std::io::Write;
 
-        if fprint == ZERO_FILLED_CHUNK.0 {
-            return Ok(Some(ZERO_FILLED_CHUNK.1.clone()));
+        if let Some(chunk) = self.known_chunks.get(&fprint) {
+            return Ok(Some(chunk.clone()));
         }
 
         let contents = fetch_from_cache(fprint);
