@@ -476,13 +476,14 @@ pub(crate) fn restore_slashes(input: &str) -> Result<Option<String>> {
     }
 }
 
-fn mangle_string(string: &str) -> String {
-    let unslashified = replace_slashes(string);
-
+/// If `input` is at least as long as `MAX_MANGLED_NAME_LENGTH`,
+/// shorten it to slightly more than `MAX_MANGLED_NAME_LENGTH`
+/// in order to guarantee the name fits in `NAME_MAX`.
+fn pseudo_uniquely_fit_string_in_filename(input: String) -> String {
     // Avoid fencepost bugs and never generate a mangled string
     // that's exactly MAX_MANGLED_NAME_LENGTH long.
-    if unslashified.as_bytes().len() < MAX_MANGLED_NAME_LENGTH {
-        return unslashified;
+    if input.as_bytes().len() < MAX_MANGLED_NAME_LENGTH {
+        return input;
     }
 
     lazy_static::lazy_static! {
@@ -492,7 +493,7 @@ fn mangle_string(string: &str) -> String {
     // This fingerprint will be printed as 2x 16 hex char strings.
     let fprint = MANGLE_PARAMS
         .fingerprinter(0)
-        .write(unslashified.as_bytes())
+        .write(input.as_bytes())
         .digest();
 
     // MAX_MANGLED_NAME_LENGTH is a bit short of the expected
@@ -506,15 +507,18 @@ fn mangle_string(string: &str) -> String {
     // MAX_MANGLED_NAME_LENGTH is comfortable less than 255, so the
     // final mangled name is both definitely longer than
     // MAX_MANGLED_NAME_LENGTH and shorter than 255.
-    let left = String::from_utf8_lossy(&unslashified.as_bytes()[..fragment_len]);
-    let right =
-        String::from_utf8_lossy(&unslashified.as_bytes()[unslashified.len() - fragment_len..]);
+    let left = String::from_utf8_lossy(&input.as_bytes()[..fragment_len]);
+    let right = String::from_utf8_lossy(&input.as_bytes()[input.len() - fragment_len..]);
     format!(
         "{}{:016x}{:016x}{}",
         left, fprint.hash[0], fprint.hash[1], right
     )
 }
 
+fn mangle_string(string: &str) -> String {
+    pseudo_uniquely_fit_string_in_filename(replace_slashes(string))
+}
+
 /// Mangles a path to an extent database into a directory name:
 /// control characters and `#` (and `%`) are percent-encoded away, and
 /// forward slashes are turned into `#`.
@@ -560,6 +564,17 @@ fn file_is_stale(path: &Path, max_age: Duration) -> Result<bool> {
     Ok(elapsed >= max_age)
 }
 
+/// Attempts to atomically rename a path and then removes it.
+fn remove_dir(path: &Path) -> std::io::Result<()> {
+    let mut new_path = path.to_path_buf();
+    new_path.set_extension(".del");
+
+    match std::fs::rename(path, &new_path) {
+        Ok(()) => std::fs::remove_dir_all(&new_path),
+        Err(_) => std::fs::remove_dir_all(path),
+    }
+}
+
 /// Attempts to delete all directories in the parent of `goal_path`
 /// except `goal_path`.
 #[instrument(err)]
@@ -581,7 +596,7 @@ fn delete_stale_directories(goal_path: &Path) -> Result<()> {
     for subdir in it.flatten() {
         if Some(subdir.file_name().as_os_str()) != goal_filename {
             parent.push(subdir.file_name());
-            if let Err(error) = std::fs::remove_dir_all(&parent) {
+            if let Err(error) = remove_dir(&parent) {
                 if error.kind() != std::io::ErrorKind::NotFound {
                     tracing::info!(?parent, %error, "failed to remove sibling");
                 }
@@ -636,7 +651,7 @@ fn delete_dangling_replication_directories(mut parent: PathBuf) -> Result<()> {
             parent.push(subdir.file_name());
             tracing::info!(%base_file, victim=?parent,
                            "deleting dangling replication directory");
-            drop_result!(std::fs::remove_dir_all(&parent),
+            drop_result!(remove_dir(&parent),
                          e => chain_info!(e, "failed to delete replication directory for missing db",
                                           %base_file, victim=?parent));
             parent.pop();
@@ -822,7 +837,10 @@ pub fn manifest_name_for_hostname_path(hostname_or: Option<&str>, path: &Path) -
         path_hash % (1 << (4 * 4)),
         string
     );
-    Ok(percent_encoding::utf8_percent_encode(&name, &ESCAPED).to_string())
+
+    Ok(pseudo_uniquely_fit_string_in_filename(
+        percent_encoding::utf8_percent_encode(&name, &ESCAPED).to_string(),
+    ))
 }
 
 /// Returns a conservatively percent-encoded URI for `path`.
