@@ -147,8 +147,11 @@ fn fetch_new_data(path: String) -> Result<Arc<Data>> {
         None => return Err(fresh_error!("manifest not found", %path)),
     };
 
-    let manifest = crate::Manifest::decode_and_validate(&*bytes, &path)
-        .map_err(|e| chain_error!(e, "failed to parse manifest file", %path))?;
+    // Use the global list of replication targets, same as the
+    // `Snapshot` below.
+    let (manifest, base) =
+        crate::Manifest::decode_and_validate(&*bytes, Default::default(), None, &path)
+            .map_err(|e| chain_error!(e, "failed to parse manifest file", %path))?;
 
     let (ctime, ctime_ns) = match &manifest.v1 {
         Some(v1) => (
@@ -164,7 +167,7 @@ fn fetch_new_data(path: String) -> Result<Arc<Data>> {
         ctime,
         ctime_ns,
         reload_queued: AtomicBool::new(false),
-        data: Snapshot::new_with_default_targets(&manifest)?,
+        data: Snapshot::new_with_default_targets(&manifest, base)?,
     });
 
     LIVE_DATA
